#include "para.h"

int exec_vp_pid_bg(pid_t *pid, const char *file, char *const args[])
{
	if ((*pid = fork()) < 0)
		return -1;
	if (!(*pid)) {
		/* child */
		execvp(file, args);
		_exit(EXIT_FAILURE);
	}
	return 1;
}

FILE *popen_read_vp_pid(pid_t *pid, const char *file, char *const args[])
{
	//pid_t pid;
	int fd[2];

	if (pipe(fd) < 0)
		return NULL;
	if ((*pid = fork()) < 0)
		return NULL;
	if (!(*pid)) {
		/* child */
		int null = open("/dev/null", O_RDONLY);
		close(fd[0]);
		if (fd[1] != STDOUT_FILENO)
			dup2(fd[1], STDOUT_FILENO);
		dup2(null, STDERR_FILENO);
		close(fd[1]);
		close(null);
		execvp(file, args);
		_exit(EXIT_FAILURE);
	}
	/* parent */
	close(fd[1]);
	return fdopen(fd[0], "r");
}

FILE *popen_read_vp(const char *file, char *const args[])
{
	pid_t pid;
	return popen_read_vp_pid(&pid, file, args);
}

FILE *popen_write_vp_pid(pid_t *pid, const char *file, char *const args[])
{
//	pid_t pid;
	int fd[2];


	if (pipe(fd) < 0)
		return NULL;
	if ((*pid = fork()) < 0)
		return NULL;
	if (!*pid) {
		/* child */
		int null = open("/dev/null", O_WRONLY);
		close(fd[1]);
		dup2(fd[0], STDIN_FILENO);
		close(fd[0]);
		//close(STDOUT_FILENO);
		//close(STDERR_FILENO);
		dup2(null, STDOUT_FILENO);
		dup2(null, STDERR_FILENO);
		close(null);
		execvp(file, args);
		_exit(EXIT_FAILURE);
	}
	/* parent */
	close(fd[0]);
	return fdopen(fd[1], "w");
}

FILE *popen_write_vp(const char *file, char *const args[])
{
	pid_t pid;
	
	return popen_write_vp_pid(&pid, file, args);
}

static void va2array(char** array, char **theargs)
{
	char **argptr = theargs;
	int i = 0;

	while (*argptr && i < 1023) {
	//printf("theargs=%s\n",*argptr);
		array[i] = *argptr;
		argptr++;
		i++;
	}
	array[i] = NULL;
	return;
}

#if 0
FILE *popen_read_pid(pid_t *pid, char **theargs)
{
	char *array[1024];
	
	va2array(array, theargs);
	return popen_read_vp_pid(pid, array[0], array);
}

FILE *popen_read(char **theargs)
{
	pid_t pid;
	return popen_read_pid(&pid, theargs);
}

FILE *popen_write_pid(pid_t *pid, char **theargs)
{
	char *array[1024];
	
	va2array(array, &theargs);
	return popen_write_vp_pid(pid, array[0], array);
}

FILE *popen_write(char *theargs)
{
	pid_t pid;
	
	return popen_write_pid(&pid, theargs);
}
#endif

FILE *popen_read_client_pid(pid_t *pid, char **cmd)
{
	char *array[1024];

	array[0] = "client";
	va2array(array + 1, cmd);
	return  popen_read_vp_pid(pid, BINDIR "/para_client", array);
}
FILE *popen_read_client(char **cmd)
{
	pid_t pid;
	return popen_read_client_pid(&pid, cmd);
}

FILE *popen_read_dbtool(char **cmd)
{
	char *array[1024];

	array[0] = "dbtool";
	va2array(array + 1, cmd);
	return  popen_read_vp(BINDIR "/para_dbtool", array);
}


int split_args(char *args, char ***argv_ptr, int delim)
{
	char *p = args;
	char **argv;
	ssize_t n = 0, i;

	while ((p = strchr(p, delim))) {
		p++;
		n++;
	}
	*argv_ptr = malloc((n + 3) * sizeof(char *));
	argv = *argv_ptr;
	i = 0;
	p = args;
//	printf("split_args: a:%s\n", p);
	while (p) {
		argv[i] = p;
	 	p = strchr(p, delim);
		if (p) {
//		printf("a:%s\n", p);
			*p = '\0';
//		printf("b:\n");
			p++;
			i++;
		}
	}
	argv[n + 1] = NULL;
	return n;
}

FILE *popen_read_client_cmdline_pid(pid_t *pid, char *cmdline)
{
	int argc;
	char **argv;

	argc = split_args(cmdline, &argv, ' ');
	if (!argv)
		return NULL;
	return popen_read_client_pid(pid, argv);
}
FILE *popen_read_client_cmdline(char *cmdline)
{
	pid_t pid;
	
	return popen_read_client_cmdline_pid(&pid, cmdline);
}

FILE *popen_read_cmdline_pid(pid_t *pid, char *cmdline)
{
	int argc;
	char **argv;
	char *tmp = strdup(cmdline);
	FILE *pipe;

	if (!tmp)
		return NULL;
	argc = split_args(tmp, &argv, ' ');
	if (!argv)
		return NULL;
	pipe = popen_read_vp_pid(pid, argv[0], argv);
	free(tmp);
	return pipe;
}

FILE *popen_write_cmdline_pid(pid_t *pid, char *cmdline)
{
	int argc;
	char **argv;
	char *tmp = strdup(cmdline);
	FILE *pipe;
	
	if (!tmp)
		return NULL;
	argc = split_args(tmp, &argv, ' ');
	if (!argv)
		return NULL;
//	fprintf(stderr, "%s arguments\n", argv[2]);
	pipe = popen_write_vp_pid(pid, argv[0], argv);
	free(tmp);
	return pipe;
}

int file_exists(const char *fn)
{
        struct stat statbuf;

        return !stat(fn, &statbuf);
}
int exec_cmdline_pid_bg(pid_t *pid, const char *cmdline)
{
	char *tmp = strdup(cmdline);
	char **argv;
	int ret;

	if (!tmp)
		return -1;
	split_args(tmp, &argv, ' ');
	if (!argv)
		return -1;
	ret = exec_vp_pid_bg(pid, argv[0], argv);
	free(tmp);
	return ret;
}
